# Copyright (c) 2014 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os_ken.controller import event as os_ken_event
from os_ken.controller import handler


class EventRowBase(os_ken_event.EventBase):
    def __init__(self, system_id, table, row, event_type):
        super(EventRowBase, self).__init__()
        self.system_id = system_id
        self.table = table
        self.row = row
        self.event_type = event_type

    def __str__(self):
        return '%s<system_id=%s table=%s, uuid=%s>' % (self.__class__.__name__,
                                                       self.system_id,
                                                       self.table,
                                                       self.row['_uuid'])


class EventRowDelete(EventRowBase):
    def __init__(self, system_id, table, row):
        super(EventRowDelete, self).__init__(system_id, table, row, 'Deleted')


class EventRowInsert(EventRowBase):
    def __init__(self, system_id, table, row):
        super(EventRowInsert, self).__init__(system_id, table, row, 'Inserted')


class EventRowUpdate(os_ken_event.EventBase):
    def __init__(self, system_id, table, old, new):
        super(EventRowUpdate, self).__init__()
        self.system_id = system_id
        self.table = table
        self.old = old
        self.new = new
        self.event_type = 'Updated'

    def __str__(self):
        return '%s<system_id=%s table=%s, uuid=%s>' % (self.__class__.__name__,
                                                       self.system_id,
                                                       self.table,
                                                       self.old['_uuid'])


class EventModifyRequest(os_ken_event.EventRequestBase):
    """ Dispatch a modify function to OVSDB

    `func` must be a callable that accepts an insert fucntion and the
    IDL.tables object. It can then modify the tables as needed. For inserts,
    specify a UUID for each insert, and return a tuple of the temporary
    UUID's. The execution of `func` will be wrapped in a single transaction
    and the reply will include a dict of temporary UUID to real UUID mappings.

    e.g.

        new_port_uuid = uuid.uuid4()

        def modify(tables, insert):
            bridges = tables['Bridge'].rows
            bridge = None
            for b in bridges:
                if b.name == 'my-bridge':
                    bridge = b

            if not bridge:
                return

            port = insert('Port', new_port_uuid)

            bridge.ports = bridge.ports + [port]

            return (new_port_uuid, )

        request = EventModifyRequest(system_id, modify)
        reply = send_request(request)

        port_uuid = reply.insert_uuids[new_port_uuid]
    """

    def __init__(self, system_id, func):
        super(EventModifyRequest, self).__init__()
        self.dst = 'OVSDB'
        self.system_id = system_id
        self.func = func

    def __str__(self):
        return '%s<system_id=%s>' % (self.__class__.__name__, self.system_id)


class EventModifyReply(os_ken_event.EventReplyBase):
    def __init__(self, system_id, status, insert_uuids, err_msg):
        self.system_id = system_id
        self.status = status
        self.insert_uuids = insert_uuids
        self.err_msg = err_msg

    def __str__(self):
        return ('%s<system_id=%s, status=%s, insert_uuids=%s, error_msg=%s>'
                % (self.__class__.__name__,
                   self.system_id,
                   self.status,
                   self.insert_uuids,
                   self.err_msg))


class EventNewOVSDBConnection(os_ken_event.EventBase):
    def __init__(self, client):
        super(EventNewOVSDBConnection, self).__init__()
        self.client = client

    def __str__(self):
        return '%s<system_id=%s>' % (self.__class__.__name__,
                                     self.client.system_id)

    @property
    def system_id(self):
        return self.client.system_id


class EventReadRequest(os_ken_event.EventRequestBase):
    def __init__(self, system_id, func):
        self.system_id = system_id
        self.func = func
        self.dst = 'OVSDB'


class EventReadReply(os_ken_event.EventReplyBase):
    def __init__(self, system_id, result, err_msg=''):
        self.system_id = system_id
        self.result = result
        self.err_msg = err_msg


class EventRowInsertedBase(EventRowInsert):
    def __init__(self, ev):
        super(EventRowInsertedBase, self).__init__(ev.system_id,
                                                   ev.table,
                                                   ev.row)


class EventRowDeletedBase(EventRowDelete):
    def __init__(self, ev):
        super(EventRowDeletedBase, self).__init__(ev.system_id,
                                                  ev.table,
                                                  ev.row)


class EventRowUpdatedBase(EventRowUpdate):
    def __init__(self, ev):
        super(EventRowUpdatedBase, self).__init__(ev.system_id,
                                                  ev.table,
                                                  ev.old,
                                                  ev.new)


class EventPortInserted(EventRowInsertedBase):
    pass


class EventPortDeleted(EventRowDeletedBase):
    pass


class EventPortUpdated(EventRowUpdatedBase):
    pass


class EventInterfaceInserted(EventRowInsertedBase):
    pass


class EventInterfaceDeleted(EventRowDeletedBase):
    pass


class EventInterfaceUpdated(EventRowUpdatedBase):
    pass


handler.register_service('os_ken.services.protocols.ovsdb.manager')



# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# vim: tabstop=4 shiftwidth=4 softtabstop=4

import copy
import logging
from struct import pack, unpack_from
import unittest

from nose.tools import ok_, eq_, raises
from os_ken.ofproto import ether
from os_ken.lib.packet.ethernet import ethernet
from os_ken.lib.packet.packet import Packet
from os_ken.lib import addrconv
from os_ken.lib.packet.slow import slow, lacp
from os_ken.lib.packet.slow import SLOW_PROTOCOL_MULTICAST
from os_ken.lib.packet.slow import SLOW_SUBTYPE_LACP
from os_ken.lib.packet.slow import SLOW_SUBTYPE_MARKER

LOG = logging.getLogger(__name__)


class Test_slow(unittest.TestCase):
    """ Test case for Slow Protocol
    """

    def setUp(self):
        self.subtype = SLOW_SUBTYPE_LACP
        self.version = lacp.LACP_VERSION_NUMBER
        self.actor_tag = lacp.LACP_TLV_TYPE_ACTOR
        self.actor_length = 20
        self.actor_system_priority = 65534
        self.actor_system = '00:07:0d:af:f4:54'
        self.actor_key = 1
        self.actor_port_priority = 65535
        self.actor_port = 1
        self.actor_state_activity = lacp.LACP_STATE_ACTIVE
        self.actor_state_timeout = lacp.LACP_STATE_LONG_TIMEOUT
        self.actor_state_aggregation = lacp.LACP_STATE_AGGREGATEABLE
        self.actor_state_synchronization = lacp.LACP_STATE_IN_SYNC
        self.actor_state_collecting = lacp.LACP_STATE_COLLECTING_ENABLED
        self.actor_state_distributing = lacp.LACP_STATE_DISTRIBUTING_ENABLED
        self.actor_state_defaulted = lacp.LACP_STATE_OPERATIONAL_PARTNER
        self.actor_state_expired = lacp.LACP_STATE_EXPIRED
        self.actor_state = (
            (self.actor_state_activity << 0) |
            (self.actor_state_timeout << 1) |
            (self.actor_state_aggregation << 2) |
            (self.actor_state_synchronization << 3) |
            (self.actor_state_collecting << 4) |
            (self.actor_state_distributing << 5) |
            (self.actor_state_defaulted << 6) |
            (self.actor_state_expired << 7))
        self.partner_tag = lacp.LACP_TLV_TYPE_PARTNER
        self.partner_length = 20
        self.partner_system_priority = 0
        self.partner_system = '00:00:00:00:00:00'
        self.partner_key = 0
        self.partner_port_priority = 0
        self.partner_port = 0
        self.partner_state_activity = 0
        self.partner_state_timeout = lacp.LACP_STATE_SHORT_TIMEOUT
        self.partner_state_aggregation = 0
        self.partner_state_synchronization = 0
        self.partner_state_collecting = 0
        self.partner_state_distributing = 0
        self.partner_state_defaulted = 0
        self.partner_state_expired = 0
        self.partner_state = (
            (self.partner_state_activity << 0) |
            (self.partner_state_timeout << 1) |
            (self.partner_state_aggregation << 2) |
            (self.partner_state_synchronization << 3) |
            (self.partner_state_collecting << 4) |
            (self.partner_state_distributing << 5) |
            (self.partner_state_defaulted << 6) |
            (self.partner_state_expired << 7))
        self.collector_tag = lacp.LACP_TLV_TYPE_COLLECTOR
        self.collector_length = 16
        self.collector_max_delay = 0
        self.terminator_tag = lacp.LACP_TLV_TYPE_TERMINATOR
        self.terminator_length = 0

        self.head_fmt = lacp._HLEN_PACK_STR
        self.head_len = lacp._HLEN_PACK_LEN
        self.act_fmt = lacp._ACTPRT_INFO_PACK_STR
        self.act_len = lacp._ACTPRT_INFO_PACK_LEN
        self.prt_fmt = lacp._ACTPRT_INFO_PACK_STR
        self.prt_len = lacp._ACTPRT_INFO_PACK_LEN
        self.col_fmt = lacp._COL_INFO_PACK_STR
        self.col_len = lacp._COL_INFO_PACK_LEN
        self.trm_fmt = lacp._TRM_PACK_STR
        self.trm_len = lacp._TRM_PACK_LEN
        self.length = lacp._ALL_PACK_LEN

        self.head_buf = pack(self.head_fmt,
                             self.subtype,
                             self.version)
        self.act_buf = pack(self.act_fmt,
                            self.actor_tag,
                            self.actor_length,
                            self.actor_system_priority,
                            addrconv.mac.text_to_bin(self.actor_system),
                            self.actor_key,
                            self.actor_port_priority,
                            self.actor_port,
                            self.actor_state)
        self.prt_buf = pack(self.prt_fmt,
                            self.partner_tag,
                            self.partner_length,
                            self.partner_system_priority,
                            addrconv.mac.text_to_bin(self.partner_system),
                            self.partner_key,
                            self.partner_port_priority,
                            self.partner_port,
                            self.partner_state)
        self.col_buf = pack(self.col_fmt,
                            self.collector_tag,
                            self.collector_length,
                            self.collector_max_delay)
        self.trm_buf = pack(self.trm_fmt,
                            self.terminator_tag,
                            self.terminator_length)

        self.buf = self.head_buf + self.act_buf + self.prt_buf + \
            self.col_buf + self.trm_buf

    def tearDown(self):
        pass

    def test_parser(self):
        slow.parser(self.buf)

    def test_not_implemented_subtype(self):
        not_implemented_buf = pack(
            slow._PACK_STR, SLOW_SUBTYPE_MARKER) + self.buf[1:]
        (instance, nexttype, last) = slow.parser(not_implemented_buf)
        assert instance is None
        assert nexttype is None
        assert last is not None

    def test_invalid_subtype(self):
        invalid_buf = b'\xff' + self.buf[1:]
        (instance, nexttype, last) = slow.parser(invalid_buf)
        assert instance is None
        assert nexttype is None
        assert last is not None


class Test_lacp(unittest.TestCase):
    """ Test case for lacp
    """

    def setUp(self):
        self.subtype = SLOW_SUBTYPE_LACP
        self.version = lacp.LACP_VERSION_NUMBER
        self.actor_tag = lacp.LACP_TLV_TYPE_ACTOR
        self.actor_length = 20
        self.actor_system_priority = 65534
        self.actor_system = '00:07:0d:af:f4:54'
        self.actor_key = 1
        self.actor_port_priority = 65535
        self.actor_port = 1
        self.actor_state_activity = lacp.LACP_STATE_ACTIVE
        self.actor_state_timeout = lacp.LACP_STATE_LONG_TIMEOUT
        self.actor_state_aggregation = lacp.LACP_STATE_AGGREGATEABLE
        self.actor_state_synchronization = lacp.LACP_STATE_IN_SYNC
        self.actor_state_collecting = lacp.LACP_STATE_COLLECTING_ENABLED
        self.actor_state_distributing = lacp.LACP_STATE_DISTRIBUTING_ENABLED
        self.actor_state_defaulted = lacp.LACP_STATE_OPERATIONAL_PARTNER
        self.actor_state_expired = lacp.LACP_STATE_EXPIRED
        self.actor_state = (
            (self.actor_state_activity << 0) |
            (self.actor_state_timeout << 1) |
            (self.actor_state_aggregation << 2) |
            (self.actor_state_synchronization << 3) |
            (self.actor_state_collecting << 4) |
            (self.actor_state_distributing << 5) |
            (self.actor_state_defaulted << 6) |
            (self.actor_state_expired << 7))
        self.partner_tag = lacp.LACP_TLV_TYPE_PARTNER
        self.partner_length = 20
        self.partner_system_priority = 0
        self.partner_system = '00:00:00:00:00:00'
        self.partner_key = 0
        self.partner_port_priority = 0
        self.partner_port = 0
        self.partner_state_activity = 0
        self.partner_state_timeout = lacp.LACP_STATE_SHORT_TIMEOUT
        self.partner_state_aggregation = 0
        self.partner_state_synchronization = 0
        self.partner_state_collecting = 0
        self.partner_state_distributing = 0
        self.partner_state_defaulted = 0
        self.partner_state_expired = 0
        self.partner_state = (
            (self.partner_state_activity << 0) |
            (self.partner_state_timeout << 1) |
            (self.partner_state_aggregation << 2) |
            (self.partner_state_synchronization << 3) |
            (self.partner_state_collecting << 4) |
            (self.partner_state_distributing << 5) |
            (self.partner_state_defaulted << 6) |
            (self.partner_state_expired << 7))
        self.collector_tag = lacp.LACP_TLV_TYPE_COLLECTOR
        self.collector_length = 16
        self.collector_max_delay = 0
        self.terminator_tag = lacp.LACP_TLV_TYPE_TERMINATOR
        self.terminator_length = 0

        self.head_fmt = lacp._HLEN_PACK_STR
        self.head_len = lacp._HLEN_PACK_LEN
        self.act_fmt = lacp._ACTPRT_INFO_PACK_STR
        self.act_len = lacp._ACTPRT_INFO_PACK_LEN
        self.prt_fmt = lacp._ACTPRT_INFO_PACK_STR
        self.prt_len = lacp._ACTPRT_INFO_PACK_LEN
        self.col_fmt = lacp._COL_INFO_PACK_STR
        self.col_len = lacp._COL_INFO_PACK_LEN
        self.trm_fmt = lacp._TRM_PACK_STR
        self.trm_len = lacp._TRM_PACK_LEN
        self.length = lacp._ALL_PACK_LEN

        self.head_buf = pack(self.head_fmt,
                             self.subtype,
                             self.version)
        self.act_buf = pack(self.act_fmt,
                            self.actor_tag,
                            self.actor_length,
                            self.actor_system_priority,
                            addrconv.mac.text_to_bin(self.actor_system),
                            self.actor_key,
                            self.actor_port_priority,
                            self.actor_port,
                            self.actor_state)
        self.prt_buf = pack(self.prt_fmt,
                            self.partner_tag,
                            self.partner_length,
                            self.partner_system_priority,
                            addrconv.mac.text_to_bin(self.partner_system),
                            self.partner_key,
                            self.partner_port_priority,
                            self.partner_port,
                            self.partner_state)
        self.col_buf = pack(self.col_fmt,
                            self.collector_tag,
                            self.collector_length,
                            self.collector_max_delay)
        self.trm_buf = pack(self.trm_fmt,
                            self.terminator_tag,
                            self.terminator_length)

        self.buf = self.head_buf + self.act_buf + self.prt_buf + \
            self.col_buf + self.trm_buf

        self.l = lacp(self.version,
                      self.actor_system_priority,
                      self.actor_system,
                      self.actor_key,
                      self.actor_port_priority,
                      self.actor_port,
                      self.actor_state_activity,
                      self.actor_state_timeout,
                      self.actor_state_aggregation,
                      self.actor_state_synchronization,
                      self.actor_state_collecting,
                      self.actor_state_distributing,
                      self.actor_state_defaulted,
                      self.actor_state_expired,
                      self.partner_system_priority,
                      self.partner_system,
                      self.partner_key,
                      self.partner_port_priority,
                      self.partner_port,
                      self.partner_state_activity,
                      self.partner_state_timeout,
                      self.partner_state_aggregation,
                      self.partner_state_synchronization,
                      self.partner_state_collecting,
                      self.partner_state_distributing,
                      self.partner_state_defaulted,
                      self.partner_state_expired,
                      self.collector_max_delay)

    def tearDown(self):
        pass

    def find_protocol(self, pkt, name):
        for p in pkt.protocols:
            if p.protocol_name == name:
                return p

    def test_init(self):
        eq_(self.subtype, self.l._subtype)
        eq_(self.version, self.l.version)
        eq_(self.actor_tag, self.l._actor_tag)
        eq_(self.actor_length, self.l._actor_length)
        eq_(self.actor_system_priority, self.l.actor_system_priority)
        eq_(self.actor_system, self.l.actor_system)
        eq_(self.actor_key, self.l.actor_key)
        eq_(self.actor_port_priority, self.l.actor_port_priority)
        eq_(self.actor_port, self.l.actor_port)
        eq_(self.actor_state_activity, self.l.actor_state_activity)
        eq_(self.actor_state_timeout, self.l.actor_state_timeout)
        eq_(self.actor_state_aggregation,
            self.l.actor_state_aggregation)
        eq_(self.actor_state_synchronization,
            self.l.actor_state_synchronization)
        eq_(self.actor_state_collecting,
            self.l.actor_state_collecting)
        eq_(self.actor_state_distributing,
            self.l.actor_state_distributing)
        eq_(self.actor_state_defaulted, self.l.actor_state_defaulted)
        eq_(self.actor_state_expired, self.l.actor_state_expired)
        eq_(self.actor_state, self.l._actor_state)
        eq_(self.partner_tag, self.l._partner_tag)
        eq_(self.partner_length, self.l._partner_length)
        eq_(self.partner_system_priority,
            self.l.partner_system_priority)
        eq_(self.partner_system, self.l.partner_system)
        eq_(self.partner_key, self.l.partner_key)
        eq_(self.partner_port_priority, self.l.partner_port_priority)
        eq_(self.partner_port, self.l.partner_port)
        eq_(self.partner_state_activity, self.l.partner_state_activity)
        eq_(self.partner_state_timeout, self.l.partner_state_timeout)
        eq_(self.partner_state_aggregation,
            self.l.partner_state_aggregation)
        eq_(self.partner_state_synchronization,
            self.l.partner_state_synchronization)
        eq_(self.partner_state_collecting,
            self.l.partner_state_collecting)
        eq_(self.partner_state_distributing,
            self.l.partner_state_distributing)
        eq_(self.partner_state_defaulted,
            self.l.partner_state_defaulted)
        eq_(self.partner_state_expired, self.l.partner_state_expired)
        eq_(self.partner_state, self.l._partner_state)
        eq_(self.collector_tag, self.l._collector_tag)
        eq_(self.collector_length, self.l._collector_length)
        eq_(self.collector_max_delay, self.l.collector_max_delay)
        eq_(self.terminator_tag, self.l._terminator_tag)
        eq_(self.terminator_length, self.l._terminator_length)

    def test_parser(self):
        _res = self.l.parser(self.buf)
        if type(_res) is tuple:
            res = _res[0]
        else:
            res = _res

        eq_(res._subtype, self.subtype)
        eq_(res.version, self.version)
        eq_(res._actor_tag, self.actor_tag)
        eq_(res._actor_length, self.actor_length)
        eq_(res.actor_system_priority, self.actor_system_priority)
        eq_(res.actor_system, self.actor_system)
        eq_(res.actor_key, self.actor_key)
        eq_(res.actor_port_priority, self.actor_port_priority)
        eq_(res.actor_port, self.actor_port)
        eq_(res.actor_state_activity, self.actor_state_activity)
        eq_(res.actor_state_timeout, self.actor_state_timeout)
        eq_(res.actor_state_aggregation, self.actor_state_aggregation)
        eq_(res.actor_state_synchronization,
            self.actor_state_synchronization)
        eq_(res.actor_state_collecting, self.actor_state_collecting)
        eq_(res.actor_state_distributing, self.actor_state_distributing)
        eq_(res.actor_state_defaulted, self.actor_state_defaulted)
        eq_(res.actor_state_expired, self.actor_state_expired)
        eq_(res._actor_state, self.actor_state)
        eq_(res._partner_tag, self.partner_tag)
        eq_(res._partner_length, self.partner_length)
        eq_(res.partner_system_priority, self.partner_system_priority)
        eq_(res.partner_system, self.partner_system)
        eq_(res.partner_key, self.partner_key)
        eq_(res.partner_port_priority, self.partner_port_priority)
        eq_(res.partner_port, self.partner_port)
        eq_(res.partner_state_activity, self.partner_state_activity)
        eq_(res.partner_state_timeout, self.partner_state_timeout)
        eq_(res.partner_state_aggregation,
            self.partner_state_aggregation)
        eq_(res.partner_state_synchronization,
            self.partner_state_synchronization)
        eq_(res.partner_state_collecting, self.partner_state_collecting)
        eq_(res.partner_state_distributing,
            self.partner_state_distributing)
        eq_(res.partner_state_defaulted, self.partner_state_defaulted)
        eq_(res.partner_state_expired, self.partner_state_expired)
        eq_(res._partner_state, self.partner_state)
        eq_(res._collector_tag, self.collector_tag)
        eq_(res._collector_length, self.collector_length)
        eq_(res.collector_max_delay, self.collector_max_delay)
        eq_(res._terminator_tag, self.terminator_tag)
        eq_(res._terminator_length, self.terminator_length)

    def test_serialize(self):
        data = bytearray()
        prev = None
        buf = self.l.serialize(data, prev)

        offset = 0
        head_res = unpack_from(self.head_fmt, buf, offset)
        offset += self.head_len
        act_res = unpack_from(self.act_fmt, buf, offset)
        offset += self.act_len
        prt_res = unpack_from(self.prt_fmt, buf, offset)
        offset += self.prt_len
        col_res = unpack_from(self.col_fmt, buf, offset)
        offset += self.col_len
        trm_res = unpack_from(self.trm_fmt, buf, offset)

        eq_(head_res[0], self.subtype)
        eq_(head_res[1], self.version)

        eq_(act_res[0], self.actor_tag)
        eq_(act_res[1], self.actor_length)
        eq_(act_res[2], self.actor_system_priority)
        eq_(act_res[3], addrconv.mac.text_to_bin(self.actor_system))
        eq_(act_res[4], self.actor_key)
        eq_(act_res[5], self.actor_port_priority)
        eq_(act_res[6], self.actor_port)
        eq_(act_res[7], self.actor_state)

        eq_(prt_res[0], self.partner_tag)
        eq_(prt_res[1], self.partner_length)
        eq_(prt_res[2], self.partner_system_priority)
        eq_(prt_res[3], addrconv.mac.text_to_bin(self.partner_system))
        eq_(prt_res[4], self.partner_key)
        eq_(prt_res[5], self.partner_port_priority)
        eq_(prt_res[6], self.partner_port)
        eq_(prt_res[7], self.partner_state)

        eq_(col_res[0], self.collector_tag)
        eq_(col_res[1], self.collector_length)
        eq_(col_res[2], self.collector_max_delay)

        eq_(trm_res[0], self.terminator_tag)
        eq_(trm_res[1], self.terminator_length)

    def _build_lacp(self):
        ethertype = ether.ETH_TYPE_SLOW
        dst = SLOW_PROTOCOL_MULTICAST
        e = ethernet(dst, self.actor_system, ethertype)
        p = Packet()

        p.add_protocol(e)
        p.add_protocol(self.l)
        p.serialize()
        return p

    def test_build_lacp(self):
        p = self._build_lacp()

        e = self.find_protocol(p, "ethernet")
        ok_(e)
        eq_(e.ethertype, ether.ETH_TYPE_SLOW)

        l = self.find_protocol(p, "lacp")
        ok_(l)

        eq_(l._subtype, self.subtype)
        eq_(l.version, self.version)
        eq_(l._actor_tag, self.actor_tag)
        eq_(l._actor_length, self.actor_length)
        eq_(l.actor_system_priority, self.actor_system_priority)
        eq_(l.actor_system, self.actor_system)
        eq_(l.actor_key, self.actor_key)
        eq_(l.actor_port_priority, self.actor_port_priority)
        eq_(l.actor_port, self.actor_port)
        eq_(l.actor_state_activity, self.actor_state_activity)
        eq_(l.actor_state_timeout, self.actor_state_timeout)
        eq_(l.actor_state_aggregation, self.actor_state_aggregation)
        eq_(l.actor_state_synchronization,
            self.actor_state_synchronization)
        eq_(l.actor_state_collecting, self.actor_state_collecting)
        eq_(l.actor_state_distributing, self.actor_state_distributing)
        eq_(l.actor_state_defaulted, self.actor_state_defaulted)
        eq_(l.actor_state_expired, self.actor_state_expired)
        eq_(l._actor_state, self.actor_state)
        eq_(l._partner_tag, self.partner_tag)
        eq_(l._partner_length, self.partner_length)
        eq_(l.partner_system_priority, self.partner_system_priority)
        eq_(l.partner_system, self.partner_system)
        eq_(l.partner_key, self.partner_key)
        eq_(l.partner_port_priority, self.partner_port_priority)
        eq_(l.partner_port, self.partner_port)
        eq_(l.partner_state_activity, self.partner_state_activity)
        eq_(l.partner_state_timeout, self.partner_state_timeout)
        eq_(l.partner_state_aggregation, self.partner_state_aggregation)
        eq_(l.partner_state_synchronization,
            self.partner_state_synchronization)
        eq_(l.partner_state_collecting, self.partner_state_collecting)
        eq_(l.partner_state_distributing,
            self.partner_state_distributing)
        eq_(l.partner_state_defaulted, self.partner_state_defaulted)
        eq_(l.partner_state_expired, self.partner_state_expired)
        eq_(l._partner_state, self.partner_state)
        eq_(l._collector_tag, self.collector_tag)
        eq_(l._collector_length, self.collector_length)
        eq_(l.collector_max_delay, self.collector_max_delay)
        eq_(l._terminator_tag, self.terminator_tag)
        eq_(l._terminator_length, self.terminator_length)

    @raises(Exception)
    def test_malformed_lacp(self):
        m_short_buf = self.buf[1:self.length]
        slow.parser(m_short_buf)

    @raises(Exception)
    def test_invalid_subtype(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.subtype = 0xff
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_version(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.version = 0xff
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_actor_tag(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.actor_tag = 0x04
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_actor_length(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.actor_length = 50
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_partner_tag(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.partner_tag = 0x01
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_partner_length(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.partner_length = 0
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_collector_tag(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.collector_tag = 0x00
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_collector_length(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.collector_length = 20
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_terminator_tag(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.terminator_tag = 0x04
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_terminator_length(self):
        invalid_lacv = copy.deepcopy(self.l)
        invalid_lacv.terminator_length = self.trm_len
        invalid_buf = invalid_lacv.serialize()
        slow.parser(invalid_buf)

    @raises(Exception)
    def test_invalid_actor_state_activity(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 2,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_timeout(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 2,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_aggregation(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 2,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_synchronization(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 2,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_collecting(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 2,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_distributing(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 2,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_defaulted(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 2,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_actor_state_expired(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 2,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_activity(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 -1,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_timeout(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 -1,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_aggregation(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 -1,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_synchronization(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 -1,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_collecting(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 -1,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_distributing(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 -1,
                 self.partner_state_defaulted,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_defaulted(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 -1,
                 self.partner_state_expired,
                 self.collector_max_delay)
        l.serialize()

    @raises(Exception)
    def test_invalid_partner_state_expired(self):
        l = lacp(self.version,
                 self.actor_system_priority,
                 self.actor_system,
                 self.actor_key,
                 self.actor_port_priority,
                 self.actor_port,
                 self.actor_state_activity,
                 self.actor_state_timeout,
                 self.actor_state_aggregation,
                 self.actor_state_synchronization,
                 self.actor_state_collecting,
                 self.actor_state_distributing,
                 self.actor_state_defaulted,
                 self.actor_state_expired,
                 self.partner_system_priority,
                 self.partner_system,
                 self.partner_key,
                 self.partner_port_priority,
                 self.partner_port,
                 self.partner_state_activity,
                 self.partner_state_timeout,
                 self.partner_state_aggregation,
                 self.partner_state_synchronization,
                 self.partner_state_collecting,
                 self.partner_state_distributing,
                 self.partner_state_defaulted,
                 -1,
                 self.collector_max_delay)
        l.serialize()

    def test_json(self):
        jsondict = self.l.to_jsondict()
        l = lacp.from_jsondict(jsondict['lacp'])
        eq_(str(self.l), str(l))



# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Parsing libpcap and reading/writing PCAP file.
Reference source: http://wiki.wireshark.org/Development/LibpcapFileFormat


                  Libpcap File Format

                +---------------------+
                |                     |
                |     Global Header   |
                |                     |
                +---------------------+
                |     Packet Header   |
                +---------------------+
                |     Packet Data     |
                +---------------------+
                |     Packet Header   |
                +---------------------+
                |     Packet Data     |
                +---------------------+
                |          ...        |
                +---------------------+
"""

import struct
import sys
import time


class PcapFileHdr(object):
    """
    Global Header
    typedef struct pcap_hdr_s {
                guint32 magic_number;   /* magic number */
                guint16 version_major;  /* major version number */
                guint16 version_minor;  /* minor version number */
                gint32  thiszone;       /* GMT to local correction */
                guint32 sigfigs;        /* accuracy of timestamps */
                guint32 snaplen;        /* max length of captured packets,
                                           in octets */
                guint32 network;        /* data link type */
    } pcap_hdr_t;

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                          Magic Number                         |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |        Version Major          |        Version Minor          |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                            Thiszone                           |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                            Sigfigs                            |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                            Snaplen                            |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                            Network                            |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
                                File Format
    """
    _FILE_HDR_FMT = '4sHHIIII'
    _FILE_HDR_FMT_BIG_ENDIAN = '>' + _FILE_HDR_FMT
    _FILE_HDR_FMT_LITTLE_ENDIAN = '<' + _FILE_HDR_FMT
    FILE_HDR_SIZE = struct.calcsize(_FILE_HDR_FMT)

    # Magic Number field is used to detect the file format itself and
    # the byte ordering.
    MAGIC_NUMBER_IDENTICAL = b'\xa1\xb2\xc3\xd4'  # Big Endian
    MAGIC_NUMBER_SWAPPED = b'\xd4\xc3\xb2\xa1'    # Little Endian

    def __init__(self, magic=MAGIC_NUMBER_SWAPPED, version_major=2,
                 version_minor=4, thiszone=0, sigfigs=0, snaplen=0,
                 network=0):
        self.magic = magic
        self.version_major = version_major
        self.version_minor = version_minor
        self.thiszone = thiszone
        self.sigfigs = sigfigs
        self.snaplen = snaplen
        self.network = network

    @classmethod
    def parser(cls, buf):
        magic_buf = buf[:4]
        if magic_buf == cls.MAGIC_NUMBER_IDENTICAL:
            # Big Endian
            fmt = cls._FILE_HDR_FMT_BIG_ENDIAN
            byteorder = 'big'
        elif magic_buf == cls.MAGIC_NUMBER_SWAPPED:
            # Little Endian
            fmt = cls._FILE_HDR_FMT_LITTLE_ENDIAN
            byteorder = 'little'
        else:
            raise struct.error('Invalid byte ordered pcap file.')

        return cls(*struct.unpack_from(fmt, buf)), byteorder

    def serialize(self):
        if sys.byteorder == 'big':
            # Big Endian
            fmt = self._FILE_HDR_FMT_BIG_ENDIAN
            self.magic = self.MAGIC_NUMBER_IDENTICAL
        else:
            # Little Endian
            fmt = self._FILE_HDR_FMT_LITTLE_ENDIAN
            self.magic = self.MAGIC_NUMBER_SWAPPED

        return struct.pack(fmt, self.magic, self.version_major,
                           self.version_minor, self.thiszone,
                           self.sigfigs, self.snaplen, self.network)


class PcapPktHdr(object):
    """
    Record (Packet) Header
    typedef struct pcaprec_hdr_s {
            guint32 ts_sec;       /* timestamp seconds */
            guint32 ts_usec;      /* timestamp microseconds */
            guint32 incl_len;     /* number of octets of packet
                                     saved in file */
            guint32 orig_len;     /* actual length of packet */
    } pcaprec_hdr_t;

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                        Timestamp Seconds                      |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                        Timestamp Microseconds                 |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                  Number of octets of saved in file            |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                        Actual length of packet                |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
                        Record (Packet) Header Format
    """

    _PKT_HDR_FMT = 'IIII'
    _PKT_HDR_FMT_BIG_ENDIAN = '>' + _PKT_HDR_FMT
    _PKT_HDR_FMT_LITTLE_ENDIAN = '<' + _PKT_HDR_FMT
    PKT_HDR_SIZE = struct.calcsize(_PKT_HDR_FMT)

    def __init__(self, ts_sec=0, ts_usec=0, incl_len=0, orig_len=0):
        self.ts_sec = ts_sec
        self.ts_usec = ts_usec
        self.incl_len = incl_len
        self.orig_len = orig_len

    @classmethod
    def parser(cls, buf, byteorder='little'):
        if not buf:
            raise IndexError('No data')

        if byteorder == 'big':
            # Big Endian
            fmt = cls._PKT_HDR_FMT_BIG_ENDIAN
        else:
            # Little Endian
            fmt = cls._PKT_HDR_FMT_LITTLE_ENDIAN

        (ts_sec, ts_usec, incl_len, orig_len) = struct.unpack_from(fmt, buf)
        hdr = cls(ts_sec, ts_usec, incl_len, orig_len)

        return hdr, buf[cls.PKT_HDR_SIZE:cls.PKT_HDR_SIZE + incl_len]

    def serialize(self):
        if sys.byteorder == 'big':
            # Big Endian
            fmt = self._PKT_HDR_FMT_BIG_ENDIAN
        else:
            # Little Endian
            fmt = self._PKT_HDR_FMT_LITTLE_ENDIAN

        return struct.pack(fmt, self.ts_sec, self.ts_usec,
                           self.incl_len, self.orig_len)


class Reader(object):
    """
    PCAP file reader

    ================ ===================================
    Argument         Description
    ================ ===================================
    file_obj         File object which reading PCAP file
                     in binary mode
    ================ ===================================

    Example of usage::

        from os_ken.lib import pcaplib
        from os_ken.lib.packet import packet

        frame_count = 0
        # iterate pcaplib.Reader that yields (timestamp, packet_data)
        # in the PCAP file
        for ts, buf in pcaplib.Reader(open('test.pcap', 'rb')):
            frame_count += 1
            pkt = packet.Packet(buf)
            print("%d, %f, %s" % (frame_count, ts, pkt))
    """

    def __init__(self, file_obj):
        self._fp = file_obj
        buf = self._fp.read(PcapFileHdr.FILE_HDR_SIZE)
        # Read only pcap file header
        self.pcap_header, self._file_byteorder = PcapFileHdr.parser(buf)
        # Read pcap data with out header
        self._pcap_body = self._fp.read()
        self._fp.close()
        self._next_pos = 0

    def __iter__(self):
        return self

    def next(self):
        try:
            pkt_hdr, pkt_data = PcapPktHdr.parser(
                self._pcap_body[self._next_pos:], self._file_byteorder)
            self._next_pos += pkt_hdr.incl_len + PcapPktHdr.PKT_HDR_SIZE

        except IndexError:
            raise StopIteration()

        return pkt_hdr.ts_sec + (pkt_hdr.ts_usec / 1e6), pkt_data

    # for Python 3 compatible
    __next__ = next


class Writer(object):
    """
    PCAP file writer

    ========== ==================================================
    Argument   Description
    ========== ==================================================
    file_obj   File object which writing PCAP file in binary mode
    snaplen    Max length of captured packets (in octets)
    network    Data link type. (e.g. 1 for Ethernet,
               see `tcpdump.org`_ for details)
    ========== ==================================================

    .. _tcpdump.org: http://www.tcpdump.org/linktypes.html

    Example of usage::

        ...
        from os_ken.lib import pcaplib


        class SimpleSwitch13(app_manager.OSKenApp):
            OFP_VERSIONS = [ofproto_v1_3.OFP_VERSION]

            def __init__(self, *args, **kwargs):
                super(SimpleSwitch13, self).__init__(*args, **kwargs)
                self.mac_to_port = {}

                # Create pcaplib.Writer instance with a file object
                # for the PCAP file
                self.pcap_writer = pcaplib.Writer(open('mypcap.pcap', 'wb'))

            ...

            @set_ev_cls(ofp_event.EventOFPPacketIn, MAIN_DISPATCHER)
            def _packet_in_handler(self, ev):
                # Dump the packet data into PCAP file
                self.pcap_writer.write_pkt(ev.msg.data)

                ...
    """

    def __init__(self, file_obj, snaplen=65535, network=1):
        self._f = file_obj
        self.snaplen = snaplen
        self.network = network
        self._write_pcap_file_hdr()

    def _write_pcap_file_hdr(self):
        pcap_file_hdr = PcapFileHdr(snaplen=self.snaplen,
                                    network=self.network)
        self._f.write(pcap_file_hdr.serialize())

    def _write_pkt_hdr(self, ts, buf_len):
        sec = int(ts)
        usec = int(round(ts % 1, 6) * 1e6) if sec != 0 else 0

        pc_pkt_hdr = PcapPktHdr(ts_sec=sec, ts_usec=usec,
                                incl_len=buf_len, orig_len=buf_len)

        self._f.write(pc_pkt_hdr.serialize())

    def write_pkt(self, buf, ts=None):
        ts = time.time() if ts is None else ts

        # Check the max length of captured packets
        buf_len = len(buf)
        if buf_len > self.snaplen:
            buf_len = self.snaplen
            buf = buf[:self.snaplen]

        self._write_pkt_hdr(ts, buf_len)

        self._f.write(buf)

    def __del__(self):
        self._f.close()



# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# This is based on the following
#     https://github.com/osrg/gobgp/test/lib/quagga.py
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

import netaddr

from . import docker_base as base

LOG = logging.getLogger(__name__)


class QuaggaBGPContainer(base.BGPContainer):

    WAIT_FOR_BOOT = 1
    SHARED_VOLUME = '/etc/quagga'

    def __init__(self, name, asn, router_id, ctn_image_name, zebra=False):
        super(QuaggaBGPContainer, self).__init__(name, asn, router_id,
                                                 ctn_image_name)
        self.shared_volumes.append((self.config_dir, self.SHARED_VOLUME))
        self.zebra = zebra
        self._create_config_debian()

    def run(self, wait=False, w_time=WAIT_FOR_BOOT):
        w_time = super(QuaggaBGPContainer,
                       self).run(wait=wait, w_time=self.WAIT_FOR_BOOT)
        return w_time

    def get_global_rib(self, prefix='', rf='ipv4'):
        rib = []
        if prefix != '':
            return self.get_global_rib_with_prefix(prefix, rf)

        out = self.vtysh('show bgp {0} unicast'.format(rf), config=False)
        if out.startswith('No BGP network exists'):
            return rib

        read_next = False

        for line in out.split('\n'):
            ibgp = False
            if line[:2] == '*>':
                line = line[2:]
                if line[0] == 'i':
                    line = line[1:]
                    ibgp = True
            elif not read_next:
                continue

            elems = line.split()

            if len(elems) == 1:
                read_next = True
                prefix = elems[0]
                continue
            elif read_next:
                nexthop = elems[0]
            else:
                prefix = elems[0]
                nexthop = elems[1]
            read_next = False

            rib.append({'prefix': prefix, 'nexthop': nexthop,
                        'ibgp': ibgp})

        return rib

    def get_global_rib_with_prefix(self, prefix, rf):
        rib = []

        lines = [line.strip() for line in self.vtysh(
            'show bgp {0} unicast {1}'.format(rf, prefix),
            config=False).split('\n')]

        if lines[0] == '% Network not in table':
            return rib

        lines = lines[2:]

        if lines[0].startswith('Not advertised'):
            lines.pop(0)  # another useless line
        elif lines[0].startswith('Advertised to non peer-group peers:'):
            lines = lines[2:]  # other useless lines
        else:
            raise Exception('unknown output format {0}'.format(lines))

        if lines[0] == 'Local':
            aspath = []
        else:
            aspath = [int(asn) for asn in lines[0].split()]

        nexthop = lines[1].split()[0].strip()
        info = [s.strip(',') for s in lines[2].split()]
        attrs = []
        if 'metric' in info:
            med = info[info.index('metric') + 1]
            attrs.append({'type': base.BGP_ATTR_TYPE_MULTI_EXIT_DISC,
                          'metric': int(med)})
        if 'localpref' in info:
            localpref = info[info.index('localpref') + 1]
            attrs.append({'type': base.BGP_ATTR_TYPE_LOCAL_PREF,
                          'value': int(localpref)})

        rib.append({'prefix': prefix, 'nexthop': nexthop,
                    'aspath': aspath, 'attrs': attrs})

        return rib

    def get_neighbor_state(self, peer):
        if peer not in self.peers:
            raise Exception('not found peer {0}'.format(peer.router_id))

        neigh_addr = self.peers[peer]['neigh_addr'].split('/')[0]

        info = [l.strip() for l in self.vtysh(
            'show bgp neighbors {0}'.format(neigh_addr),
            config=False).split('\n')]

        if not info[0].startswith('BGP neighbor is'):
            raise Exception('unknown format')

        idx1 = info[0].index('BGP neighbor is ')
        idx2 = info[0].index(',')
        n_addr = info[0][idx1 + len('BGP neighbor is '):idx2]
        if n_addr == neigh_addr:
            idx1 = info[2].index('= ')
            state = info[2][idx1 + len('= '):]
            if state.startswith('Idle'):
                return base.BGP_FSM_IDLE
            elif state.startswith('Active'):
                return base.BGP_FSM_ACTIVE
            elif state.startswith('Established'):
                return base.BGP_FSM_ESTABLISHED
            else:
                return state

        raise Exception('not found peer {0}'.format(peer.router_id))

    def send_route_refresh(self):
        self.vtysh('clear ip bgp * soft', config=False)

    def create_config(self):
        zebra = 'no'
        self._create_config_bgp()
        if self.zebra:
            zebra = 'yes'
            self._create_config_zebra()
        self._create_config_daemons(zebra)

    def _create_config_debian(self):
        c = base.CmdBuffer()
        c << 'vtysh_enable=yes'
        c << 'zebra_options="  --daemon -A 127.0.0.1"'
        c << 'bgpd_options="   --daemon -A 127.0.0.1"'
        c << 'ospfd_options="  --daemon -A 127.0.0.1"'
        c << 'ospf6d_options=" --daemon -A ::1"'
        c << 'ripd_options="   --daemon -A 127.0.0.1"'
        c << 'ripngd_options=" --daemon -A ::1"'
        c << 'isisd_options="  --daemon -A 127.0.0.1"'
        c << 'babeld_options=" --daemon -A 127.0.0.1"'
        c << 'watchquagga_enable=yes'
        c << 'watchquagga_options=(--daemon)'
        with open('{0}/debian.conf'.format(self.config_dir), 'w') as f:
            LOG.info("[%s's new config]", self.name)
            LOG.info(str(c))
            f.writelines(str(c))

    def _create_config_daemons(self, zebra='no'):
        c = base.CmdBuffer()
        c << 'zebra=%s' % zebra
        c << 'bgpd=yes'
        c << 'ospfd=no'
        c << 'ospf6d=no'
        c << 'ripd=no'
        c << 'ripngd=no'
        c << 'isisd=no'
        c << 'babeld=no'
        with open('{0}/daemons'.format(self.config_dir), 'w') as f:
            LOG.info("[%s's new config]", self.name)
            LOG.info(str(c))
            f.writelines(str(c))

    def _create_config_bgp(self):

        c = base.CmdBuffer()
        c << 'hostname bgpd'
        c << 'password zebra'
        c << 'router bgp {0}'.format(self.asn)
        c << 'bgp router-id {0}'.format(self.router_id)
        if any(info['graceful_restart'] for info in self.peers.values()):
            c << 'bgp graceful-restart'

        version = 4
        for peer, info in self.peers.items():
            version = netaddr.IPNetwork(info['neigh_addr']).version
            n_addr = info['neigh_addr'].split('/')[0]
            if version == 6:
                c << 'no bgp default ipv4-unicast'

            c << 'neighbor {0} remote-as {1}'.format(n_addr, peer.asn)
            if info['is_rs_client']:
                c << 'neighbor {0} route-server-client'.format(n_addr)
            for typ, p in info['policies'].items():
                c << 'neighbor {0} route-map {1} {2}'.format(n_addr, p['name'],
                                                             typ)
            if info['passwd']:
                c << 'neighbor {0} password {1}'.format(n_addr, info['passwd'])
            if info['passive']:
                c << 'neighbor {0} passive'.format(n_addr)
            if version == 6:
                c << 'address-family ipv6 unicast'
                c << 'neighbor {0} activate'.format(n_addr)
                c << 'exit-address-family'

        for route in self.routes.values():
            if route['rf'] == 'ipv4':
                c << 'network {0}'.format(route['prefix'])
            elif route['rf'] == 'ipv6':
                c << 'address-family ipv6 unicast'
                c << 'network {0}'.format(route['prefix'])
                c << 'exit-address-family'
            else:
                raise Exception(
                    'unsupported route faily: {0}'.format(route['rf']))

        if self.zebra:
            if version == 6:
                c << 'address-family ipv6 unicast'
                c << 'redistribute connected'
                c << 'exit-address-family'
            else:
                c << 'redistribute connected'

        for name, policy in self.policies.items():
            c << 'access-list {0} {1} {2}'.format(name, policy['type'],
                                                  policy['match'])
            c << 'route-map {0} permit 10'.format(name)
            c << 'match ip address {0}'.format(name)
            c << 'set metric {0}'.format(policy['med'])

        c << 'debug bgp as4'
        c << 'debug bgp fsm'
        c << 'debug bgp updates'
        c << 'debug bgp events'
        c << 'log file {0}/bgpd.log'.format(self.SHARED_VOLUME)

        with open('{0}/bgpd.conf'.format(self.config_dir), 'w') as f:
            LOG.info("[%s's new config]", self.name)
            LOG.info(str(c))
            f.writelines(str(c))

    def _create_config_zebra(self):
        c = base.CmdBuffer()
        c << 'hostname zebra'
        c << 'password zebra'
        c << 'log file {0}/zebra.log'.format(self.SHARED_VOLUME)
        c << 'debug zebra packet'
        c << 'debug zebra kernel'
        c << 'debug zebra rib'
        c << ''

        with open('{0}/zebra.conf'.format(self.config_dir), 'w') as f:
            LOG.info("[%s's new config]", self.name)
            LOG.info(str(c))
            f.writelines(str(c))

    def vtysh(self, cmd, config=True):
        if not isinstance(cmd, list):
            cmd = [cmd]
        cmd = ' '.join("-c '{0}'".format(c) for c in cmd)
        if config:
            return self.exec_on_ctn(
                "vtysh -d bgpd -c 'en' -c 'conf t' -c "
                "'router bgp {0}' {1}".format(self.asn, cmd),
                capture=True)
        else:
            return self.exec_on_ctn("vtysh -d bgpd {0}".format(cmd),
                                    capture=True)

    def reload_config(self):
        daemon = []
        daemon.append('bgpd')
        if self.zebra:
            daemon.append('zebra')
        for d in daemon:
            cmd = '/usr/bin/pkill {0} -SIGHUP'.format(d)
            self.exec_on_ctn(cmd, capture=True)


class RawQuaggaBGPContainer(QuaggaBGPContainer):
    def __init__(self, name, config, ctn_image_name,
                 zebra=False):
        asn = None
        router_id = None
        for line in config.split('\n'):
            line = line.strip()
            if line.startswith('router bgp'):
                asn = int(line[len('router bgp'):].strip())
            if line.startswith('bgp router-id'):
                router_id = line[len('bgp router-id'):].strip()
        if not asn:
            raise Exception('asn not in quagga config')
        if not router_id:
            raise Exception('router-id not in quagga config')
        self.config = config
        super(RawQuaggaBGPContainer, self).__init__(name, asn, router_id,
                                                    ctn_image_name, zebra)

    def create_config(self):
        with open(os.path.join(self.config_dir, 'bgpd.conf'), 'w') as f:
            LOG.info("[%s's new config]", self.name)
            LOG.info(self.config)
            f.writelines(self.config)



# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import struct
import unittest

from nose.tools import eq_
from nose.tools import ok_
from nose.tools import raises
from os_ken.ofproto import ether
from os_ken.ofproto import inet
from os_ken.lib.packet import ethernet
from os_ken.lib.packet import packet
from os_ken.lib.packet import ipv4
from os_ken.lib.packet import vlan
from os_ken.lib.packet import pbb


LOG = logging.getLogger(__name__)


class Test_itag(unittest.TestCase):

    pcp = 3
    dei = 0
    uca = 1
    sid = 16770000
    data = pcp << 29 | dei << 28 | uca << 27 | sid
    buf = struct.pack(pbb.itag._PACK_STR, data)
    it = pbb.itag(pcp, dei, uca, sid)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_init(self):
        eq_(self.pcp, self.it.pcp)
        eq_(self.dei, self.it.dei)
        eq_(self.uca, self.it.uca)
        eq_(self.sid, self.it.sid)

    def test_parser(self):
        _res = pbb.itag.parser(self.buf)
        if type(_res) is tuple:
            res = _res[0]
        else:
            res = _res
        eq_(res.pcp, self.pcp)
        eq_(res.dei, self.dei)
        eq_(res.uca, self.uca)
        eq_(res.sid, self.sid)

    def test_serialize(self):
        data = bytearray()
        prev = None
        buf = self.it.serialize(data, prev)
        res = struct.unpack(pbb.itag._PACK_STR, buf)
        eq_(res[0], self.data)

    def _build_itag(self):
        b_src_mac = '00:07:0d:af:f4:54'
        b_dst_mac = '00:00:00:00:00:00'
        b_ethertype = ether.ETH_TYPE_8021AD
        e1 = ethernet.ethernet(b_dst_mac, b_src_mac, b_ethertype)

        b_pcp = 0
        b_cfi = 0
        b_vid = 32
        b_ethertype = ether.ETH_TYPE_8021Q
        bt = vlan.svlan(b_pcp, b_cfi, b_vid, b_ethertype)

        c_src_mac = '11:11:11:11:11:11'
        c_dst_mac = 'aa:aa:aa:aa:aa:aa'
        c_ethertype = ether.ETH_TYPE_8021AD
        e2 = ethernet.ethernet(c_dst_mac, c_src_mac, c_ethertype)

        s_pcp = 0
        s_cfi = 0
        s_vid = 32
        s_ethertype = ether.ETH_TYPE_8021Q
        st = vlan.svlan(s_pcp, s_cfi, s_vid, s_ethertype)

        c_pcp = 0
        c_cfi = 0
        c_vid = 32
        c_ethertype = ether.ETH_TYPE_IP
        ct = vlan.vlan(c_pcp, c_cfi, c_vid, c_ethertype)

        version = 4
        header_length = 20
        tos = 0
        total_length = 24
        identification = 0x8a5d
        flags = 0
        offset = 1480
        ttl = 64
        proto = inet.IPPROTO_ICMP
        csum = 0xa7f2
        src = '131.151.32.21'
        dst = '131.151.32.129'
        option = b'TEST'
        ip = ipv4.ipv4(version, header_length, tos, total_length,
                       identification, flags, offset, ttl, proto, csum,
                       src, dst, option)

        p = packet.Packet()

        p.add_protocol(e1)
        p.add_protocol(bt)
        p.add_protocol(self.it)
        p.add_protocol(e2)
        p.add_protocol(st)
        p.add_protocol(ct)
        p.add_protocol(ip)
        p.serialize()

        return p

    def test_build_itag(self):
        p = self._build_itag()

        e = p.get_protocols(ethernet.ethernet)
        ok_(e)
        ok_(isinstance(e, list))
        eq_(e[0].ethertype, ether.ETH_TYPE_8021AD)
        eq_(e[1].ethertype, ether.ETH_TYPE_8021AD)

        sv = p.get_protocols(vlan.svlan)
        ok_(sv)
        ok_(isinstance(sv, list))
        eq_(sv[0].ethertype, ether.ETH_TYPE_8021Q)
        eq_(sv[1].ethertype, ether.ETH_TYPE_8021Q)

        it = p.get_protocol(pbb.itag)
        ok_(it)

        v = p.get_protocol(vlan.vlan)
        ok_(v)
        eq_(v.ethertype, ether.ETH_TYPE_IP)

        ip = p.get_protocol(ipv4.ipv4)
        ok_(ip)

        eq_(it.pcp, self.pcp)
        eq_(it.dei, self.dei)
        eq_(it.uca, self.uca)
        eq_(it.sid, self.sid)

    @raises(Exception)
    def test_malformed_itag(self):
        m_short_buf = self.buf[1:pbb.itag._MIN_LEN]
        pbb.itag.parser(m_short_buf)

    def test_json(self):
        jsondict = self.it.to_jsondict()
        it = pbb.itag.from_jsondict(jsondict['itag'])
        eq_(str(self.it), str(it))



# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2013 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import sys
import unittest
from nose.tools import eq_
from nose.tools import ok_

from os_ken.ofproto import ofproto_v1_2
from os_ken.ofproto import ofproto_v1_3
from os_ken.ofproto import ofproto_v1_2_parser
from os_ken.ofproto import ofproto_v1_3_parser

from os_ken.lib import addrconv
from os_ken.tests import test_lib
from struct import unpack


class Test_Parser_Compat(unittest.TestCase):
    def __init__(self, methodName):
        print('init %s' % methodName)
        super(Test_Parser_Compat, self).__init__(methodName)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _test(self, name, ofpp):
        ofp = {
            ofproto_v1_2_parser: ofproto_v1_2,
            ofproto_v1_3_parser: ofproto_v1_3,
        }[ofpp]

        in_port = 987654321
        eth_src = 'aa:bb:cc:dd:ee:ff'
        ipv4_src = '192.0.2.9'
        ipv6_src = 'fe80::f00b:a4ff:feef:5d8f'

        old_in_port = in_port
        old_eth_src = addrconv.mac.text_to_bin(eth_src)
        old_ipv4_src = unpack('!I', addrconv.ipv4.text_to_bin(ipv4_src))[0]
        old_ipv6_src = list(unpack('!8H',
                                   addrconv.ipv6.text_to_bin(ipv6_src)))

        def check(o):
            check_old(o)
            check_new(o)

        def check_old(o):
            # old api
            def get_field(m, t):
                for f in m.fields:
                    if isinstance(f, t):
                        return f
            get_value = lambda m, t: get_field(m, t).value

            eq_(get_value(o, ofpp.MTInPort), old_in_port)
            eq_(get_value(o, ofpp.MTEthSrc), old_eth_src)
            eq_(get_value(o, ofpp.MTIPV4Src), old_ipv4_src)
            eq_(get_value(o, ofpp.MTIPv6Src), old_ipv6_src)

        def check_new(o):
            # new api
            eq_(o['in_port'], in_port)
            eq_(o['eth_src'], eth_src)
            eq_(o['ipv4_src'], ipv4_src)
            eq_(o['ipv6_src'], ipv6_src)

        # ensure that old and new api produces the same thing

        # old api
        old = ofpp.OFPMatch()
        old.set_in_port(old_in_port)
        old.set_dl_src(old_eth_src)
        old.set_ipv4_src(old_ipv4_src)
        old.set_ipv6_src(old_ipv6_src)

        old_buf = bytearray()
        old.serialize(old_buf, 0)

        # note: you can't inspect an object composed with the old set_XXX api
        # before serialize().
        check_old(old)

        # another variant of old api; originally it was intended to be
        # internal but actually used in the field.  eg. LINC l2_switch_v1_3.py
        old2 = ofpp.OFPMatch()
        old2.append_field(ofp.OXM_OF_IN_PORT, old_in_port)
        old2.append_field(ofp.OXM_OF_ETH_SRC, old_eth_src)
        old2.append_field(ofp.OXM_OF_IPV4_SRC, old_ipv4_src)
        old2.append_field(ofp.OXM_OF_IPV6_SRC, old_ipv6_src)
        check_old(old2)

        old2_buf = bytearray()
        old2.serialize(old2_buf, 0)

        # new api
        new = ofpp.OFPMatch(in_port=in_port, eth_src=eth_src,
                            ipv4_src=ipv4_src, ipv6_src=ipv6_src)
        check_new(new)

        new_buf = bytearray()
        new.serialize(new_buf, 0)
        eq_(new_buf, old_buf)
        eq_(new_buf, old2_buf)

        old_jsondict = old.to_jsondict()
        old2_jsondict = old2.to_jsondict()
        new_jsondict = new.to_jsondict()
        eq_(new_jsondict, old_jsondict)
        eq_(new_jsondict, old2_jsondict)

        eq_(str(new), str(old))
        eq_(str(new), str(old2))

        # a parsed object can be inspected by old and new api

        check(ofpp.OFPMatch.parser(six.binary_type(new_buf), 0))
        check(ofpp.OFPMatch.from_jsondict(list(new_jsondict.values())[0]))


def _add_tests():
    import functools
    import itertools

    ofpps = [ofproto_v1_2_parser, ofproto_v1_3_parser]
    for ofpp in ofpps:
        mod = ofpp.__name__.split('.')[-1]
        method_name = 'test_' + mod + '_ofpmatch_compat'

        def _run(self, name, ofpp):
            print('processing %s ...' % name)
            if six.PY3:
                self._test(self, name, ofpp)
            else:
                self._test(name, ofpp)
        print('adding %s ...' % method_name)
        f = functools.partial(_run, name=method_name,
                              ofpp=ofpp)
        test_lib.add_method(Test_Parser_Compat, method_name, f)


_add_tests()
